@@ -8,7 +8,6 @@ from narwhals._dask.expr import DaskExpr
 if TYPE_CHECKING:
     import dask.dataframe.dask_expr as dx  # noqa: F401
 
-    from narwhals._compliant.typing import ScalarKwargs
     from narwhals._dask.dataframe import DaskLazyFrame  # noqa: F401
 
 
@@ -18,16 +17,10 @@ class DaskSelectorNamespace(LazySelectorNamespace["DaskLazyFrame", "dx.Series"])
         return DaskSelector
 
 
-class DaskSelector(CompliantSelector["DaskLazyFrame", "dx.Series"], DaskExpr):  # type: ignore[misc]
-    _depth: int = 0
-    _scalar_kwargs: ScalarKwargs = {}  # noqa: RUF012
-    _function_name: str = "selector"
-
+class DaskSelector(CompliantSelector["DaskLazyFrame", "dx.Series"], DaskExpr):  # pyright: ignore[reportInvalidTypeArguments]
     def _to_expr(self) -> DaskExpr:
         return DaskExpr(
             self._call,
-            depth=self._depth,
-            function_name=self._function_name,
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=self._alias_output_names,
             version=self._version,
