@@ -289,7 +289,6 @@ class PolarsDataFrame(PolarsBaseFrame[pl.DataFrame]):
     collect: Method[CompliantDataFrameAny]
     estimated_size: Method[int | float]
     gather_every: Method[Self]
-    item: Method[Any]
     iter_rows: Method[Iterator[tuple[Any, ...]] | Iterator[Mapping[str, Any]]]
     is_unique: Method[PolarsSeries]
     row: Method[tuple[Any, ...]]
@@ -659,6 +658,20 @@ class PolarsDataFrame(PolarsBaseFrame[pl.DataFrame]):
         except Exception as e:  # noqa: BLE001  # pragma: no cover
             raise catch_polars_exception(e) from None
 
+    def item(self, row: int | None, column: int | str | None) -> Any:
+        if (
+            self._backend_version < (1, 36)
+            and row is None
+            and column is None
+            and (shape := self.shape) != (1, 1)
+        ):
+            msg = (
+                'can only call `.item()` without "row" or "column" values if the '
+                f"DataFrame has a single element; shape={shape!r}"
+            )
+            raise ValueError(msg)
+        return self.native.item(row=row, column=column)
+
 
 class PolarsLazyFrame(PolarsBaseFrame[pl.LazyFrame]):
     sink_parquet: Method[None]
