@@ -299,6 +299,9 @@ class PolarsExpr:
         result = self.native.mode()
         return self._with_native(result.first() if keep == "any" else result)
 
+    def any_value(self, *, ignore_nulls: bool) -> Self:
+        return self.drop_nulls().first() if ignore_nulls else self.first()
+
     @property
     def dt(self) -> PolarsExprDateTimeNamespace:
         return PolarsExprDateTimeNamespace(self)
