@@ -5,14 +5,22 @@ from decimal import Decimal
 from functools import wraps
 from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, overload
 
+from narwhals import plugins
 from narwhals._constants import EPOCH, MS_PER_SECOND
-from narwhals._namespace import (
+from narwhals._native import (
     is_native_arrow,
     is_native_pandas_like,
     is_native_polars,
     is_native_spark_like,
 )
-from narwhals._utils import Implementation, Version
+from narwhals._utils import (
+    Implementation,
+    Version,
+    has_native_namespace,
+    is_compliant_dataframe,
+    is_compliant_lazyframe,
+    is_compliant_series,
+)
 from narwhals.dependencies import (
     get_dask_expr,
     get_numpy,
@@ -30,13 +38,23 @@ from narwhals.dependencies import (
 )
 
 if TYPE_CHECKING:
+    from typing_extensions import Unpack
+
+    from narwhals._translate import (
+        AllowAny,
+        AllowLazy,
+        AllowSeries,
+        ExcludeSeries,
+        OnlySeries,
+        PassThroughUnknown,
+    )
     from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
     from narwhals.typing import (
         DataFrameT,
+        Frame,
         IntoDataFrameT,
         IntoFrame,
-        IntoFrameT,
         IntoLazyFrameT,
         IntoSeries,
         IntoSeriesT,
@@ -56,8 +74,8 @@ def to_native(
 ) -> IntoDataFrameT: ...
 @overload
 def to_native(
-    narwhals_object: LazyFrame[IntoFrameT], *, pass_through: Literal[False] = ...
-) -> IntoFrameT: ...
+    narwhals_object: LazyFrame[IntoLazyFrameT], *, pass_through: Literal[False] = ...
+) -> IntoLazyFrameT: ...
 @overload
 def to_native(
     narwhals_object: Series[IntoSeriesT], *, pass_through: Literal[False] = ...
@@ -68,26 +86,15 @@ def to_native(narwhals_object: Any, *, pass_through: bool) -> Any: ...
 
 def to_native(
     narwhals_object: DataFrame[IntoDataFrameT]
-    | LazyFrame[IntoFrameT]
+    | LazyFrame[IntoLazyFrameT]
     | Series[IntoSeriesT],
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
-) -> IntoDataFrameT | IntoFrameT | IntoSeriesT | Any:
+    pass_through: bool = False,
+) -> IntoDataFrameT | IntoLazyFrameT | IntoSeriesT | Any:
     """Convert Narwhals object to native one.
 
     Arguments:
         narwhals_object: Narwhals object.
-        strict: Determine what happens if `narwhals_object` isn't a Narwhals class
-
-            - `True` (default): raise an error
-            - `False`: pass object through as-is
-
-            *Deprecated* (v1.13.0)
-
-            Please use `pass_through` instead. Note that `strict` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         pass_through: Determine what happens if `narwhals_object` isn't a Narwhals class
 
             - `False` (default): raise an error
@@ -96,14 +103,9 @@ def to_native(
     Returns:
         Object of class that user started with.
     """
-    from narwhals._utils import validate_strict_and_pass_though
     from narwhals.dataframe import BaseFrame
     from narwhals.series import Series
 
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
-    )
-
     if isinstance(narwhals_object, BaseFrame):
         return narwhals_object._compliant_frame._native_frame
     if isinstance(narwhals_object, Series):
@@ -116,152 +118,42 @@ def to_native(
 
 
 @overload
-def from_native(native_object: SeriesT, **kwds: Any) -> SeriesT: ...
-
-
+def from_native(native_object: SeriesT, **kwds: Unpack[OnlySeries]) -> SeriesT: ...
 @overload
-def from_native(native_object: DataFrameT, **kwds: Any) -> DataFrameT: ...
-
-
-@overload
-def from_native(native_object: LazyFrameT, **kwds: Any) -> LazyFrameT: ...
-
-
+def from_native(native_object: SeriesT, **kwds: Unpack[AllowSeries]) -> SeriesT: ...
 @overload
 def from_native(
-    native_object: IntoDataFrameT | IntoSeriesT,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[True],
-    series_only: Literal[False] = ...,
-    allow_series: Literal[True],
-) -> DataFrame[IntoDataFrameT] | Series[IntoSeriesT]: ...
-
-
+    native_object: DataFrameT, **kwds: Unpack[ExcludeSeries]
+) -> DataFrameT: ...
+# Closer to *intended* than https://github.com/narwhals-dev/narwhals/issues/3226
+@overload
+def from_native(native_object: LazyFrameT, **kwds: Unpack[AllowLazy]) -> LazyFrameT: ...
 @overload
 def from_native(
-    native_object: IntoDataFrameT,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
+    native_object: IntoDataFrameT, **kwds: Unpack[ExcludeSeries]
 ) -> DataFrame[IntoDataFrameT]: ...
-
-
 @overload
 def from_native(
-    native_object: T,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
-) -> T: ...
-
-
-@overload
-def from_native(
-    native_object: IntoDataFrameT,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[True],
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
-) -> DataFrame[IntoDataFrameT]: ...
-
-
-@overload
-def from_native(
-    native_object: T,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[True],
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
-) -> T: ...
-
-
-@overload
-def from_native(
-    native_object: IntoFrameT | IntoLazyFrameT | IntoSeriesT,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: Literal[True],
-) -> DataFrame[IntoFrameT] | LazyFrame[IntoLazyFrameT] | Series[IntoSeriesT]: ...
-
-
-@overload
-def from_native(
-    native_object: IntoSeriesT,
-    *,
-    pass_through: Literal[True],
-    eager_only: Literal[False] = ...,
-    series_only: Literal[True],
-    allow_series: None = ...,
+    native_object: IntoSeriesT, **kwds: Unpack[OnlySeries]
 ) -> Series[IntoSeriesT]: ...
-
-
-# NOTE: Seems like `mypy` is giving a false positive
-# Following this advice will introduce overlapping overloads?
-# > note: Flipping the order of overloads will fix this error
 @overload
-def from_native(  # type: ignore[overload-overlap]
-    native_object: IntoLazyFrameT,
-    *,
-    pass_through: Literal[False] = ...,
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
+def from_native(
+    native_object: IntoSeriesT, **kwds: Unpack[AllowSeries]
+) -> Series[IntoSeriesT]: ...
+@overload
+def from_native(
+    native_object: IntoLazyFrameT, **kwds: Unpack[AllowLazy]
 ) -> LazyFrame[IntoLazyFrameT]: ...
-
-
 @overload
 def from_native(
-    native_object: IntoDataFrameT,
-    *,
-    pass_through: Literal[False] = ...,
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
-) -> DataFrame[IntoDataFrameT]: ...
-
-
+    native_object: IntoDataFrameT | IntoSeriesT, **kwds: Unpack[AllowSeries]
+) -> DataFrame[IntoDataFrameT] | Series[IntoSeriesT]: ...
 @overload
 def from_native(
-    native_object: IntoDataFrameT,
-    *,
-    pass_through: Literal[False] = ...,
-    eager_only: Literal[True],
-    series_only: Literal[False] = ...,
-    allow_series: None = ...,
-) -> DataFrame[IntoDataFrameT]: ...
-
-
+    native_object: IntoDataFrameT | IntoLazyFrameT | IntoSeriesT, **kwds: Unpack[AllowAny]
+) -> DataFrame[IntoDataFrameT] | LazyFrame[IntoLazyFrameT] | Series[IntoSeriesT]: ...
 @overload
-def from_native(
-    native_object: IntoFrame | IntoSeries,
-    *,
-    pass_through: Literal[False] = ...,
-    eager_only: Literal[False] = ...,
-    series_only: Literal[False] = ...,
-    allow_series: Literal[True],
-) -> DataFrame[Any] | LazyFrame[Any] | Series[Any]: ...
-
-
-@overload
-def from_native(
-    native_object: IntoSeriesT,
-    *,
-    pass_through: Literal[False] = ...,
-    eager_only: Literal[False] = ...,
-    series_only: Literal[True],
-    allow_series: None = ...,
-) -> Series[IntoSeriesT]: ...
-
-
+def from_native(native_object: T, **kwds: Unpack[PassThroughUnknown]) -> T: ...
 # All params passed in as variables
 @overload
 def from_native(
@@ -272,18 +164,19 @@ def from_native(
     series_only: bool,
     allow_series: bool | None,
 ) -> Any: ...
-
-
-def from_native(  # noqa: D417
-    native_object: IntoLazyFrameT | IntoFrameT | IntoSeriesT | IntoFrame | IntoSeries | T,
+def from_native(
+    native_object: IntoLazyFrameT
+    | IntoDataFrameT
+    | IntoSeriesT
+    | IntoFrame
+    | IntoSeries
+    | T,
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
+    pass_through: bool = False,
     eager_only: bool = False,
     series_only: bool = False,
     allow_series: bool | None = None,
-    **kwds: Any,
-) -> LazyFrame[IntoLazyFrameT] | DataFrame[IntoFrameT] | Series[IntoSeriesT] | T:
+) -> LazyFrame[IntoLazyFrameT] | DataFrame[IntoDataFrameT] | Series[IntoSeriesT] | T:
     """Convert `native_object` to Narwhals Dataframe, Lazyframe, or Series.
 
     Arguments:
@@ -293,19 +186,9 @@ def from_native(  # noqa: D417
             - a Dataframe / Lazyframe / Series supported by Narwhals (pandas, Polars, PyArrow, ...)
             - an object which implements `__narwhals_dataframe__`, `__narwhals_lazyframe__`,
               or `__narwhals_series__`
-        strict: Determine what happens if the object can't be converted to Narwhals
-
-            - `True` or `None` (default): raise an error
-            - `False`: pass object through as-is
-
-            *Deprecated* (v1.13.0)
-
-            Please use `pass_through` instead. Note that `strict` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         pass_through: Determine what happens if the object can't be converted to Narwhals
 
-            - `False` or `None` (default): raise an error
+            - `False` (default): raise an error
             - `True`: pass object through as-is
         eager_only: Whether to only allow eager objects
 
@@ -324,15 +207,6 @@ def from_native(  # noqa: D417
         DataFrame, LazyFrame, Series, or original object, depending
             on which combination of parameters was passed.
     """
-    from narwhals._utils import validate_strict_and_pass_though
-
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
-    )
-    if kwds:
-        msg = f"from_native() got an unexpected keyword argument {next(iter(kwds))!r}"
-        raise TypeError(msg)
-
     return _from_native_impl(  # type: ignore[no-any-return]
         native_object,
         pass_through=pass_through,
@@ -344,23 +218,65 @@ def from_native(  # noqa: D417
     )
 
 
+def _translate_if_compliant(  # noqa: C901,PLR0911
+    compliant_object: Any,
+    *,
+    pass_through: bool = False,
+    eager_only: bool = False,
+    # Interchange-level was removed after v1
+    eager_or_interchange_only: bool,
+    series_only: bool,
+    allow_series: bool | None,
+    version: Version,
+) -> Any:
+    if is_compliant_dataframe(compliant_object):
+        if series_only:
+            if not pass_through:
+                msg = "Cannot only use `series_only` with dataframe"
+                raise TypeError(msg)
+            return compliant_object
+        return version.dataframe(
+            compliant_object.__narwhals_dataframe__()._with_version(version), level="full"
+        )
+    if is_compliant_lazyframe(compliant_object):
+        if series_only:
+            if not pass_through:
+                msg = "Cannot only use `series_only` with lazyframe"
+                raise TypeError(msg)
+            return compliant_object
+        if eager_only or eager_or_interchange_only:
+            if not pass_through:
+                msg = "Cannot only use `eager_only` or `eager_or_interchange_only` with lazyframe"
+                raise TypeError(msg)
+            return compliant_object
+        return version.lazyframe(
+            compliant_object.__narwhals_lazyframe__()._with_version(version), level="full"
+        )
+    if is_compliant_series(compliant_object):
+        if not allow_series:
+            if not pass_through:
+                msg = "Please set `allow_series=True` or `series_only=True`"
+                raise TypeError(msg)
+            return compliant_object
+        return version.series(
+            compliant_object.__narwhals_series__()._with_version(version), level="full"
+        )
+    # Object wasn't compliant, can't translate here.
+    return None
+
+
 def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
     native_object: Any,
     *,
     pass_through: bool = False,
     eager_only: bool = False,
     # Interchange-level was removed after v1
-    eager_or_interchange_only: bool = False,
-    series_only: bool = False,
-    allow_series: bool | None = None,
+    eager_or_interchange_only: bool,
+    series_only: bool,
+    allow_series: bool | None,
     version: Version,
 ) -> Any:
-    from narwhals._utils import (
-        _supports_dataframe_interchange,
-        is_compliant_dataframe,
-        is_compliant_lazyframe,
-        is_compliant_series,
-    )
+    from narwhals._interchange.dataframe import supports_dataframe_interchange
     from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
 
@@ -380,41 +296,21 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         raise ValueError(msg)
 
     # Extensions
-    if is_compliant_dataframe(native_object):
-        if series_only:
-            if not pass_through:
-                msg = "Cannot only use `series_only` with dataframe"
-                raise TypeError(msg)
-            return native_object
-        return version.dataframe(
-            native_object.__narwhals_dataframe__()._with_version(version), level="full"
-        )
-    elif is_compliant_lazyframe(native_object):
-        if series_only:
-            if not pass_through:
-                msg = "Cannot only use `series_only` with lazyframe"
-                raise TypeError(msg)
-            return native_object
-        if eager_only or eager_or_interchange_only:
-            if not pass_through:
-                msg = "Cannot only use `eager_only` or `eager_or_interchange_only` with lazyframe"
-                raise TypeError(msg)
-            return native_object
-        return version.lazyframe(
-            native_object.__narwhals_lazyframe__()._with_version(version), level="full"
-        )
-    elif is_compliant_series(native_object):
-        if not allow_series:
-            if not pass_through:
-                msg = "Please set `allow_series=True` or `series_only=True`"
-                raise TypeError(msg)
-            return native_object
-        return version.series(
-            native_object.__narwhals_series__()._with_version(version), level="full"
+    if (
+        translated := _translate_if_compliant(
+            native_object,
+            pass_through=pass_through,
+            eager_only=eager_only,
+            eager_or_interchange_only=eager_or_interchange_only,
+            series_only=series_only,
+            allow_series=allow_series,
+            version=version,
         )
+    ) is not None:
+        return translated
 
     # Polars
-    elif is_native_polars(native_object):
+    if is_native_polars(native_object):
         if series_only and not is_polars_series(native_object):
             if not pass_through:
                 msg = f"Cannot only use `series_only` with {type(native_object).__qualname__}"
@@ -439,7 +335,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # PandasLike
-    elif is_native_pandas_like(native_object):
+    if is_native_pandas_like(native_object):
         if is_pandas_like_dataframe(native_object):
             if series_only:
                 if not pass_through:
@@ -458,7 +354,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # PyArrow
-    elif is_native_arrow(native_object):
+    if is_native_arrow(native_object):
         if is_pyarrow_table(native_object):
             if series_only:
                 if not pass_through:
@@ -477,7 +373,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # Dask
-    elif is_dask_dataframe(native_object):
+    if is_dask_dataframe(native_object):
         if series_only:
             if not pass_through:
                 msg = "Cannot only use `series_only` with dask DataFrame"
@@ -501,7 +397,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # DuckDB
-    elif is_duckdb_relation(native_object):
+    if is_duckdb_relation(native_object):
         if eager_only or series_only:  # pragma: no cover
             if not pass_through:
                 msg = "Cannot only use `series_only=True` or `eager_only=False` with DuckDBPyRelation"
@@ -514,7 +410,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # Ibis
-    elif is_ibis_table(native_object):
+    if is_ibis_table(native_object):
         if eager_only or series_only:  # pragma: no cover
             if not pass_through:
                 msg = "Cannot only use `series_only=True` or `eager_only=False` with ibis.Table"
@@ -527,7 +423,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         )
 
     # PySpark
-    elif is_native_spark_like(native_object):  # pragma: no cover
+    if is_native_spark_like(native_object):  # pragma: no cover
         ns_spark = version.namespace.from_native_object(native_object)
         if series_only or eager_only or eager_or_interchange_only:
             if not pass_through:
@@ -540,7 +436,7 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
         return ns_spark.compliant.from_native(native_object).to_narwhals()
 
     # Interchange protocol
-    elif _supports_dataframe_interchange(native_object):
+    if supports_dataframe_interchange(native_object):
         from narwhals._interchange.dataframe import InterchangeFrame
 
         if eager_only or series_only:
@@ -564,15 +460,27 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
             raise TypeError(msg)
         return Version.V1.dataframe(InterchangeFrame(native_object), level="interchange")
 
-    elif not pass_through:
-        msg = f"Expected pandas-like dataframe, Polars dataframe, or Polars lazyframe, got: {type(native_object)}"
+    compliant_object = plugins.from_native(native_object, version)
+    if compliant_object is not None:
+        return _translate_if_compliant(
+            compliant_object,
+            pass_through=pass_through,
+            eager_only=eager_only,
+            eager_or_interchange_only=eager_or_interchange_only,
+            series_only=series_only,
+            allow_series=allow_series,
+            version=version,
+        )
+
+    if not pass_through:
+        msg = f"Unsupported dataframe type, got: {type(native_object)}"
+        if hint := plugins._show_suggestions(type(native_object)):  # pragma: no cover
+            msg += "\n\n" + hint
         raise TypeError(msg)
     return native_object
 
 
-def get_native_namespace(
-    *obj: DataFrame[Any] | LazyFrame[Any] | Series[Any] | IntoFrame | IntoSeries,
-) -> Any:
+def get_native_namespace(*obj: Frame | Series[Any] | IntoFrame | IntoSeries) -> Any:
     """Get native namespace from object.
 
     Arguments:
@@ -605,28 +513,19 @@ def get_native_namespace(
 
 
 def _get_native_namespace_single_obj(
-    obj: DataFrame[Any] | LazyFrame[Any] | Series[Any] | IntoFrame | IntoSeries,
+    obj: Frame | Series[Any] | IntoFrame | IntoSeries,
 ) -> Any:
-    from contextlib import suppress
-
-    from narwhals._utils import has_native_namespace
-
-    with suppress(TypeError, AssertionError):
-        return Version.MAIN.namespace.from_native_object(
-            obj
-        ).implementation.to_native_namespace()
-
     if has_native_namespace(obj):
         return obj.__native_namespace__()
-    msg = f"Could not get native namespace from object of type: {type(obj)}"
-    raise TypeError(msg)
+    return Version.MAIN.namespace.from_native_object(
+        obj
+    ).implementation.to_native_namespace()
 
 
 def narwhalify(
     func: Callable[..., Any] | None = None,
     *,
-    strict: bool | None = None,
-    pass_through: bool | None = None,
+    pass_through: bool = True,
     eager_only: bool = False,
     series_only: bool = False,
     allow_series: bool | None = True,
@@ -643,20 +542,10 @@ def narwhalify(
 
     Arguments:
         func: Function to wrap in a `from_native`-`to_native` block.
-        strict: Determine what happens if the object can't be converted to Narwhals
-
-            *Deprecated* (v1.13.0)
-
-            Please use `pass_through` instead. Note that `strict` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
-
-            - `True` or `None` (default): raise an error
-            - `False`: pass object through as-is
         pass_through: Determine what happens if the object can't be converted to Narwhals
 
-            - `False` or `None` (default): raise an error
-            - `True`: pass object through as-is
+            - `False`: raise an error
+            - `True` (default): pass object through as-is
         eager_only: Whether to only allow eager objects
 
             - `False` (default): don't require `native_object` to be eager
@@ -688,11 +577,6 @@ def narwhalify(
         ... def agnostic_group_by_sum(df):
         ...     return df.group_by("a").agg(nw.col("b").sum())
     """
-    from narwhals._utils import validate_strict_and_pass_though
-
-    pass_through = validate_strict_and_pass_though(
-        strict, pass_through, pass_through_default=True, emit_deprecation_warning=True
-    )
 
     def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
         @wraps(func)
@@ -737,9 +621,8 @@ def narwhalify(
 
     if func is None:
         return decorator
-    else:
-        # If func is not None, it means the decorator is used without arguments
-        return decorator(func)
+    # If func is not None, it means the decorator is used without arguments
+    return decorator(func)
 
 
 def to_py_scalar(scalar_like: Any) -> Any:
@@ -748,9 +631,6 @@ def to_py_scalar(scalar_like: Any) -> Any:
     Arguments:
         scalar_like: Scalar-like value.
 
-    Returns:
-        Python scalar.
-
     Raises:
         ValueError: If the object is not convertible to a scalar.
 
